#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2019 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a Python plug-in for GIMP.
#

# Current version number: 0.03

# Version 0.01 Adds kerning paths to custom fonts
# It has options for adding kerning pair paths, adjusting the pairs, and displaying kerned characters
# Version 0.02 Added kerning path extension .png to kerning path names in order to be compatible with other custom font painting plug-ins
# Version 0.03 Added option to use pixel based kerning adjustments
# Version 0.03b Updated kerning paths for common characters

from gimpfu import *
#from array import array
import math
import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)


def do_stuff(image, layer, do_kern, kern_style, kern_only, use_pixels, kern_g1_amount, kern_g2_amount, A_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern, do_user_kern, kern_string, do_kern_single, single_letter, bg_col, do_finalize):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
    do_all_examples = False	
	
    if do_kern:
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
					
        loop, u_case_loop = 0,0
        # Get the list of layer ids
        layer_names_list = image.layers
        
        while loop < num_layers:
            # check that upper-case characters exist
            visi_name = pdb.gimp_item_get_name(layer_names_list[loop])
            visi_name = visi_name.decode('utf-8')
            if (len(visi_name) == 5 and ".png" in visi_name): 
                if visi_name[0].isupper() == True: u_case_loop += 1
            loop += 1
        if u_case_loop == 0:
            pdb.gimp_message("Cannot create kerning paths. There are no upper-case characters.")
        elif u_case_loop == 1:
            pdb.gimp_message("Some kerning paths cannot create be created because one upper-case character is missing.")
       
        elif u_case_loop > 1 and u_case_loop <26:
            pdb.gimp_message("Some kerning paths cannot be created because there are " + str(26 - l_case_loop) + " missing upper-case characters.")
            
    if do_kern: # if the user opts to add kerning paths
	
        do_all_examples = False
        do_kern_single = False
        do_user_kern = False
	
        # get lists of vectors and layers ids in this image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        vector_list = image.vectors
        layer_list = image.layers
    
        if num_layers>0: #check that the file actually does contain layers
            found_layer = True
        else:
            found_layer = False
			
        if found_layer == False: # Report no paths
                old_handler = pdb.gimp_message_get_handler()		
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message("There are no layers, associated with this image.")	
                pdb.gimp_message_set_handler(old_handler)
        else:
		
            # get lists of vector ids in image
            num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
            vector_list = image.vectors
            loop = 0
            while loop<num_vectors and kern_only == 0 and kern_style>0:
                vector_id = vector_list[loop]
                active_vector_name = pdb.gimp_item_get_name(vector_id)
                active_vector_name = active_vector_name.decode('utf-8')
                if len(active_vector_name)==2:
                    if active_vector_name[0].isupper() and active_vector_name[1].isupper():
                        pdb.gimp_image_remove_vectors(image, vector_list[loop])
                loop += 1
		
		       
            loop_ptr = 1
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
            loop = num_layers - loop_ptr
            layer_names_list = image.layers
	
            loop = 0
			
            kerning_group_1 = "BDEFHIKLMNPR"
            kerning_group_2 = "CGOQ"       	
        	
				
            kerning_style_part_one = ["A",0,0,"B",0,0,"C",0,0,"D",0,0,"E",0,0,"F",0,0,"G",0,0,"H",0,0,
            "I",0,0,"J",0,0,"K",0,0,"L",0,0,"M",0,0,"N",0,0,"O",0,0,"P",0,0,"Q",0,0,"R",0,0,"S",0,0,"T",0,0,"U",0,0,"V",0,0,"W",0,0,"X",0,0,"Y",0,0,"Z",0,0]
			

            if kern_style == 0:
			
			
                style_A = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_B = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_C = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_D = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_E = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_F = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_G = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_H = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_I = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_J = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_K = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_L = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_M = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_N = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_O = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_P = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_Q = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_R = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_S = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_T = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_U = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_V = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_W = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_X = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_Y = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]
                style_Z = ["A",10.0,"B",10.0,"C",10.0,"D",10.0,"E",10.0,"F",10.0,"G",10.0,"H",10.0,"I",10.0,"J",10.0,"K",10.0,"L",10.0,"M",10.0,"N",10.0,"O",10.0,"P",10.0,"Q",10.0,"R",10.0,"S",10.0,"T",10.0,"U",10.0,"V",10.0,"W",10.0,"X",10.0,"Y",10.0,"Z",10.0]


				
            if kern_style == 1:
                # Use character layer widths 
                style_A = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_B = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"H",100.0,"I",100.0,"J",100.0,"L",100.0,"O",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"Y",100.0]
                style_C = ["A",100.0,"C",100.0,"D",100.0,"E",100.0,"H",100.0,"I",100.0,"K",100.0,"L",100.0,"O",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"Y",100.0,"Z",100.0]
                style_D = ["A",100.0,"D",100.0,"E",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"L",100.0,"M",100.0,"O",100.0,"R",100.0,"S",100.0,"U",100.0,"V",100.0,"W",100.0,"Y",100.0]
                style_E = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_F = ["A",100.0,"E",100.0,"F",100.0,"G",100.0,"I",100.0,"L",100.0,"O",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"Y",100.0]
                style_G = ["A",100.0,"E",100.0,"G",100.0,"H",100.0,"I",100.0,"L",100.0,"N",100.0,"O",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"W",100.0,"Y",100.0]		
                style_H = ["A",100.0,"E",100.0,"I",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"Y",100.0]
                style_I = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_J = ["A",100.0,"E",100.0,"I",100.0,"K",100.0,"O",100.0,"S",100.0,"U",100.0,"Y",100.0]
                style_K = ["A",100.0,"E",100.0,"H",100.0,"I",100.0,"L",100.0,"N",100.0,"O",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"W",100.0,"Y",100.0]
                style_L = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"I",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"Y",100.0]
                style_M = ["A",100.0,"B",100.0,"E",100.0,"F",100.0,"G",100.0,"I",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"S",100.0,"T",100.0,"U",100.0,"Y",100.0]
                style_N = ["A",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0]
                style_O = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_P = ["A",100.0,"E",100.0,"F",100.0,"H",100.0,"I",100.0,"L",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"W",100.0,"Y",100.0]
                style_Q = ["A",100.0,"E",100.0,"I",100.0,"O",100.0,"R",100.0,"U",100.0,"W",100.0]
                style_R = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"R",100.0,"Q",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"Y",100.0,"Z",100.0]
                style_S = ["A",100.0,"C",100.0,"E",100.0,"H",100.0,"I",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"S",100.0,"T",100.0,"U",100.0,"W",100.0,"Y",100.0]
                style_T = ["A",100.0,"C",100.0,"E",100.0,"H",100.0,"I",100.0,"L",100.0,"M",100.0,"O",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"W",100.0,"Y",100.0,"Z",100.0]
                style_U = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_V = ["A",100.0,"E",100.0,"H",100.0,"I",100.0,"L",100.0,"O",100.0,"R",100.0,"S",100.0,"U",100.0,"V",100.0,"W",100.0,"Y",100.0]
                style_W = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"H",100.0,"I",100.0,"K",100.0,"L",100.0,"N",100.0,"O",100.0,"P",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"X",100.0,"Y",100.0]
                style_X = ["A",100.0,"C",100.0,"E",100.0,"H",100.0,"I",100.0,"L",100.0,"O",100.0,"P",100.0,"Q",100.0,"S",100.0,"T",100.0,"Y",100.0]
                style_Y = ["A",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"Z",100.0]
                style_Z = ["A",100.0,"E",100.0,"I",100.0,"L",100.0,"O",100.0,"S",100.0,"U",100.0,"Y",100.0,"Z",100.0]
			
            if kern_style == 2:
                # Sans Serif
                style_A = ["A",112.8,"B",100.0,"C",92.8,"D",100.0,"E",100.0,"F",100.0,"G",92.8,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",92.8,"P",100.0,"Q",92.8,"R",100.0,"S",100.0,"T",76.6,"U",100.0,"V",76.6,"W",80.3,"X",100.0,"Y",76.6,"Z",100.0]
                style_B = ["A",92.8,"B",89.1,"D",100.0,"E",89.1,"H",100.0,"I",89.1,"J",100.0,"L",89.1,"O",92.8,"R",89.1,"S",92.8,"T",100.0,"U",89.1,"Y",100.0]
                style_C = ["A",100.0,"C",100.0,"E",90.8,"H",90.8,"I",90.8,"K",100.0,"L",90.8,"O",100.0,"Q",100.0,"R",90.8,"S",100.0,"T",100.0,"U",90.8,"Y",100.0,"Z",90.8]
                style_D = ["A",90.7,"D",87.9,"E",87.9,"G",100.0,"H",87.9,"I",87.9,"J",100.0,"L",90.8,"O",100.0,"R",87.9,"S",96.0,"U",87.9,"V",100.0,"W",96.0,"Y",100.0]
                style_E = ["A",104.6,"B",87.0,"C",96.6,"D",87.0,"E",87.0,"F",87.0,"G",96.6,"H",87.0,"I",87.0,"J",83.0,"K",87.0,"L",87.0,"M",87.0,"N",87.0,"O",96.6,"P",87.0,"Q",96.6,"R",87.0,"S",96.6,"T",104.6,"U",87.0,"V",100.6,"W",100.6,"X",96.6,"Y",100.6,"Z",96.6]
                style_F = ["A",81.7,"E",91.2,"F",91.2,"G",100.0,"I",91.2,"L",91.2,"O",95.6,"R",91.2,"S",95.6,"T",100.0,"U",91.2,"Y",106.1]
                style_G = ["A",100.0,"E",90.2,"G",100.0,"H",90.2,"I",90.2,"L",90.2,"N",90.2,"O",100.0,"R",90.2,"S",100.0,"T",100.0,"U",90.2,"W",100.0,"Y",106.1]
                style_H = ["A",100.0,"E",87.8,"I",87.8,"L",100.0,"M",100.0,"N",100.0,"O",90.4,"R",100.0,"S",90.4,"T",100.0,"U",87.8,"Y",99.5]
                style_I = ["A",100.0,"B",69.5,"C",82.1,"D",69.5,"E",69.5,"F",69.5,"G",82.1,"H",69.5,"J",47.7,"K",69.5,"L",69.5,"M",100.0,"N",69.5,"O",82.1,"P",69.5,"Q",82.1,"R",69.5,"S",69.5,"T",100.0,"U",69.5,"V",100.0,"W",82.1,"X",82.1,"Y",100.0,"Z",82.1]
                style_J = ["A",91.1,"E",69.3,"I",69.3,"O",78.2,"S",78.2,"U",69.3,"Y",100.0]
                style_K = ["A",114.4,"E",100.0,"H",100.0,"I",100.0,"L",100.0,"N",100.0,"O",89.9,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"W",100.0,"Y",100.0]
                style_L = ["A",113.9,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"I",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"R",100.0,"S",103.5,"T",69.7,"U",100.0,"V",77.0,"W",100.0,"Y",69.7]
                style_M = ["A",100.0,"B",89.6,"E",89.6,"F",91.2,"G",100.0,"I",89.6,"L",89.6,"M",89.6,"N",89.6,"O",92.6,"P",89.6,"S",91.7,"T",100.0,"U",89.6,"Y",100.0]
                style_N = ["A",100.0,"C",93.5,"D",88.3,"E",100.0,"F",100.0,"G",93.5,"I",100.0,"J",81.5,"K",88.3,"L",88.3,"M",100.0,"N",88.3,"O",93.5,"P",88.3,"R",88.3,"S",90.9,"T",100.0,"U",88.3,"V",100.0,"W",95.8,"X",93.5,"Y",100.0,"Z",93.5]
                style_O = ["A",92.6,"B",92.6,"C",98.3,"D",92.6,"E",92.6,"F",92.6,"G",98.3,"H",92.6,"I",92.6,"J",86.0,"K",92.6,"L",92.6,"M",92.6,"N",92.6,"O",98.3,"P",92.6,"Q",98.3,"R",92.6,"S",94.8,"T",94.8,"U",92.6,"V",92.6,"W",92.6,"X",86.0,"Y",90.9,"Z",90.9]
                style_P = ["A",81.4,"E",94.9,"F",94.9,"H",94.9,"I",94.9,"L",94.9,"N",100.0,"O",100.6,"P",94.9,"R",94.9,"S",100.6,"T",100.6,"U",94.9,"Y",97.4]
                style_Q = ["A",94.8,"E",100.0,"I",100.0,"O",100.0,"U",92.6,"W",100.0]
                style_R = ["A",109.1,"B",100.0,"C",88.9,"D",96.3,"E",96.3,"F",96.3,"G",88.9,"H",96.3,"I",96.3,"K",96.3,"L",96.3,"M",96.3,"N",96.3,"O",88.9,"P",96.3,"R",96.3,"S",96.3,"T",92.9,"U",100.0,"V",100.0,"Y",100.0,"Z",106.5]
                style_S = ["A",96.0,"C",96.0,"E",100.0,"H",100.0,"I",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",96.0,"P",100.0,"Q",100.0,"S",92.9,"T",100.0,"U",100.0,"W",96.0,"Y",100.0]
                style_T = ["A",79.6,"C",100.0,"E",98.4,"H",98.4,"I",98.4,"L",98.4,"O",95.3,"R",98.4,"S",102.5,"T",112.9,"U",98.4,"W",109.4,"Y",115.7,"Z",106.6]
                style_U = ["A",92.5,"B",90.4,"C",95.7,"D",90.4,"E",90.4,"F",90.4,"G",95.7,"H",90.4,"I",90.4,"J",80.8,"K",90.4,"L",90.4,"M",90.4,"N",90.4,"O",95.7,"P",90.4,"Q",95.7,"R",90.4,"S",92.5,"T",100.0,"U",90.4,"V",100.0,"W",97.9,"X",95.7,"Y",102.4,"Z",95.7]
                style_V = ["A",76.6,"E",99.1,"H",99.1,"I",99.1,"L",100.0,"O",92.9,"R",99.1,"S",96.6,"U",99.1,"V",100.0,"Y",113.4]
                style_W = ["A",87.4,"B",100.0,"C",100.0,"D",100.0,"E",97.4,"F",100.0,"H",97.4,"I",97.4,"K",100.0,"N",100.0,"O",94.1,"P",100.0,"R",97.4,"S",97.4,"T",104.7,"U",97.4,"Y",107.1]
                style_X = ["A",112.7,"C",85.8,"E",96.0,"H",96.0,"I",96.0,"O",85.8,"P",96.0,"S",96.0,"T",104.0,"Y",106.2]
                style_Y = ["A",73.9,"D",108.3,"E",108.3,"F",100.0,"G",86.8,"I",108.3,"K",108.3,"L",108.3,"M",108.3,"N",108.3,"O",86.8,"R",108.3,"S",96.0,"T",108.3,"U",108.3,"Z",104.6]
                style_Z = ["A",106.2,"E",92.4,"I",92.4,"L",92.4,"O",92.4,"S",100.0,"U",92.4,"Y",104.0,"Z",98.3]


            if kern_style == 3:
                # Serif Style
                style_A = ["A",111.5,"B",110.2,"C",87.8,"D",110.2,"E",110.2,"F",110.2,"G",87.8,"H",110.2,"I",109.2,"J",109.2,"K",110.2,"L",110.2,"M",110.2,"N",107.4,"O",87.8,"P",110.2,"Q",87.8,"R",110.2,"S",103.0,"T",89.1,"U",85.0,"V",65.1,"W",70.0,"X",111.5,"Y",82.4,"Z",110.9]
                style_B = ["A",93.0,"B",94.5,"D",96.0,"E",94.5,"H",94.5,"I",93.5,"J",101.0,"L",94.5,"O",99.0,"R",94.5,"S",97.5,"T",100.0,"U",94.0,"Y",87.3]
                style_C = ["A",95.0,"C",103.8,"E",105.0,"H",105.0,"I",103.8,"K",105.0,"L",105.0,"O",103.8,"Q",103.8,"R",105.0,"S",99.0,"T",103.8,"U",106.8,"Y",106.3,"Z",96.5]
                style_D = ["A",88.9,"D",93.3,"E",93.3,"G",103.9,"H",93.3,"I",92.4,"J",94.9,"L",93.3,"O",103.9,"R",93.3,"S",99.1,"U",94.7,"V",87.8,"W",90.1,"Y",87.8]
                style_E = ["A",103.5,"B",102.2,"C",104.9,"D",102.2,"E",102.2,"F",102.2,"G",104.9,"H",102.2,"I",100.8,"J",104.9,"K",102.2,"L",102.2,"M",102.2,"N",104.9,"O",104.9,"P",102.2,"Q",104.9,"R",102.2,"S",106.3,"T",99.7,"U",103.8,"V",103.5,"W",102.2,"X",101.4,"Y",96.5,"Z",102.7]
                style_F = ["A",74.1,"E",109.6,"F",109.6,"G",106.5,"I",108.3,"L",109.6,"O",106.5,"R",109.6,"S",102.5,"T",108.3,"U",103.8,"Y",112.3]
                style_G = ["A",100.7,"E",98.8,"G",105.5,"H",98.8,"I",98.2,"L",98.8,"N",101.6,"O",105.5,"R",98.8,"S",93.1,"T",97.7,"U",100.9,"W",99.8,"Y",101.6]
                style_H = ["A",110.2,"E",108.3,"I",107.6,"L",108.3,"M",108.3,"N",108.3,"O",99.3,"R",108.3,"S",101.4,"T",106.9,"U",110.4,"Y",109.9]
                style_I = ["A",120.5,"B",116.5,"C",96.5,"D",116.5,"E",116.5,"F",116.5,"G",96.5,"H",116.5,"I",115.0,"J",116.5,"K",116.5,"L",116.5,"M",116.5,"N",122.5,"O",96.5,"P",116.5,"Q",96.5,"R",116.5,"S",102.0,"T",112.5,"U",121.0,"V",120.0,"W",118.5,"X",119.0,"Y",120.0,"Z",119.0]
                style_J = ["A",91.0,"E",118.8,"I",117.5,"O",95.3,"S",94.4,"U",122.2,"Y",121.8]
                style_K = ["A",112.8,"E",110.9,"H",110.9,"I",110.3,"L",110.9,"N",108.7,"O",91.3,"R",110.9,"S",104.3,"T",97.9,"U",101.4,"W",100.2,"Y",100.9]
                style_L = ["A",102.5,"B",100.3,"C",104.1,"D",100.3,"E",100.3,"F",100.3,"G",104.1,"I",100.3,"K",100.3,"L",100.3,"M",100.3,"N",97.8,"O",104.1,"P",100.3,"R",100.3,"S",101.1,"T",76.8,"U",95.6,"V",68.9,"W",74.4,"Y",68.4]
                style_M = ["A",108.1,"B",106.6,"E",106.6,"F",106.6,"G",96.1,"I",106.2,"L",106.6,"M",100.9,"N",108.8,"O",96.1,"P",106.6,"S",101.1,"T",104.5,"U",108.2,"Y",108.1]
                style_N = ["A",96.6,"C",97.5,"D",109.1,"E",109.1,"F",109.1,"G",97.5,"I",108.4,"J",98.3,"K",109.1,"L",109.1,"M",109.1,"N",111.8,"O",97.5,"P",109.1,"R",109.1,"S",102.5,"T",107.3,"U",111.1,"V",110.7,"W",110.0,"X",109.1,"Y",110.7]
                style_O = ["A",88.0,"B",94.2,"C",103.9,"D",94.2,"E",94.2,"F",94.2,"G",103.9,"H",94.2,"I",91.9,"J",94.2,"K",94.2,"L",94.2,"M",94.2,"N",90.5,"O",103.9,"P",94.2,"Q",103.9,"R",94.2,"S",99.3,"T",101.6,"U",94.5,"V",87.3,"W",88.0,"X",88.0,"Y",88.0,"Z",97.7]
                style_P = ["A",72.5,"E",91.0,"F",91.0,"H",91.0,"I",90.1,"L",91.0,"N",94.0,"O",103.0,"P",91.0,"R",91.0,"S",100.6,"T",106.3,"U",93.4,"Y",91.3]
                style_Q = ["A",88.0,"E",94.2,"I",91.9,"O",103.9,"U",87.5,"W",88.0]
                style_R = ["A",113.8,"B",111.8,"C",88.9,"D",111.8,"E",111.8,"F",111.8,"G",88.9,"H",111.8,"I",111.1,"K",111.8,"L",111.8,"M",111.8,"N",109.4,"O",88.9,"P",111.8,"R",111.8,"S",104.7,"T",90.4,"U",89.7,"V",80.0,"Y",82.5,"Z",113.1]
                style_S = ["A",97.6,"C",100.3,"E",97.0,"H",97.0,"I",97.0,"K",97.0,"L",97.0,"M",97.0,"N",101.5,"O",100.3,"P",97.0,"Q",100.3,"S",95.8,"T",94.3,"U",99.1,"W",99.1,"Y",99.1]
                style_T = ["A",87.7,"C",105.4,"E",108.2,"H",108.2,"I",107.4,"L",108.2,"O",105.4,"R",108.2,"S",102.2,"T",107.4,"U",111.4,"W",110.1,"Y",110.1,"Z",99.5]
                style_U = ["A",89.1,"B",109.7,"C",95.8,"D",109.7,"E",109.7,"F",109.7,"G",95.8,"H",109.7,"I",109.0,"J",98.6,"K",109.7,"L",109.7,"M",109.7,"N",112.2,"O",95.8,"P",109.7,"Q",95.8,"R",109.7,"S",99.3,"T",107.6,"U",111.8,"V",111.3,"W",110.6,"X",109.7,"Y",111.3,"Z",101.2]
                style_V = ["A",71.1,"E",109.7,"H",109.7,"I",109.0,"L",109.7,"O",90.3,"R",109.7,"S",99.3,"U",111.8,"V",111.3,"Y",111.3]
                style_W = ["A",82.0,"B",107.8,"C",93.6,"D",107.8,"E",107.8,"F",107.8,"H",107.8,"I",107.2,"K",107.8,"N",109.7,"O",93.6,"P",107.8,"R",107.8,"S",99.8,"T",106.0,"U",109.4,"Y",109.0]
                style_X = ["A",112.2,"C",92.4,"E",109.7,"H",109.7,"I",109.0,"O",92.4,"P",109.7,"S",103.0,"T",105.5,"Y",109.7]
                style_Y = ["A",82.2,"D",109.2,"E",109.2,"F",109.2,"G",86.8,"I",108.5,"K",109.2,"L",109.2,"M",109.2,"N",112.0,"O",86.8,"R",109.2,"S",97.0,"T",107.4,"U",111.3,"Z",101.2]
                style_Z = ["A",105.7,"E",106.3,"I",105.7,"L",106.3,"O",102.7,"S",99.7,"U",109.8,"Y",108.2,"Z",104.9]

            if kern_style == 4:
                # All characters available
                style_A = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_B = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_C = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_D = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_E = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_F = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_G = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_H = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_I = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_J = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_K = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_L = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_M = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_N = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_O = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_P = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_Q = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_R = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_S = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_T = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_U = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_V = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_W = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_X = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_Y = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]
                style_Z = ["A",100.0,"B",100.0,"C",100.0,"D",100.0,"E",100.0,"F",100.0,"G",100.0,"H",100.0,"I",100.0,"J",100.0,"K",100.0,"L",100.0,"M",100.0,"N",100.0,"O",100.0,"P",100.0,"Q",100.0,"R",100.0,"S",100.0,"T",100.0,"U",100.0,"V",100.0,"W",100.0,"X",100.0,"Y",100.0,"Z",100.0]


            #if kern_style>0:
            kerning_style_part_two = [style_A,style_B,style_C,style_D,style_E,style_F,style_G,
            style_H,style_I,style_J,style_K,style_L,style_M,style_N,style_O,style_P,style_Q,
            style_R,style_S,style_T,style_U,style_V,style_W,style_X,style_Y,style_Z]
			
            if kern_only == 0:
                for kerning_loop in range(0,len(kerning_style_part_one),3):
                    layer_exists = pdb.gimp_image_get_layer_by_name(image, kerning_style_part_one[kerning_loop] + ".png")
				
                    if layer_exists:
                        kerning_character = kerning_style_part_one[kerning_loop]
                        kerning_width = layer_exists.width
                        kerning_y = int(layer_exists.height/2)
					
			
                        for kerning_character_loop in range(0,len(kerning_style_part_two[kerning_loop/3] ),2):
                            kerning_character_2 = kerning_style_part_two[kerning_loop/3][kerning_character_loop]
                            kerning_character_2_exists = pdb.gimp_image_get_layer_by_name(image, kerning_character_2 + ".png")
                            if kerning_character_2_exists: 
                                new_vector_name = kerning_character + kerning_character_2 + ".png" ####
                                if kern_style == 0:
                                    kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair path
                                    if kerning_vector>=0: 
                                
                                        NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(kerning_vector) # get vector information for kerning path width
                                        ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                                        kerning_target_width = pdb.gimp_vectors_stroke_get_length(kerning_vector,ActivePath_StrokeID,0.01)
                                    else:
                                        dummy = 0
                                        kerning_target_width = 0
                                        #pdb.gimp_message("No vector found")
                                else:
                                    kerning_target_width = float((kerning_width/100.00)) * float((kerning_style_part_two[kerning_loop/3][kerning_character_loop+1]))
                                    kerning_target_width = int(kerning_target_width)
                                if kerning_target_width > 0:
                                    kerning_target_width += fn_check_kern_adjustments(use_pixels, kerning_character_2, kerning_target_width, kerning_group_1, kerning_group_2, kern_g1_amount, kern_g2_amount, A_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern)
                                    new_vector_name = kerning_character + kerning_character_2 + ".png"
                                    kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair paths
                                    if kerning_vector>=0: # replace any existing character kerning path with a new version of the path
                                        pdb.gimp_image_remove_vectors(image, kerning_vector)
                                    new_vector_id = pdb.gimp_vectors_new(image, new_vector_name)
                                    pdb.gimp_image_insert_vectors(image, new_vector_id, None, -1)
                                    points1 = [0,kerning_y] * 3
                                    points2 = [kerning_target_width,kerning_y] * 3
                                    points = points1 + points2
                                    new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
            else:
                kerning_loop = (kern_only * 3) - 3
                layer_exists = pdb.gimp_image_get_layer_by_name(image, kerning_style_part_one[kerning_loop] + ".png")
                if layer_exists:
                    kerning_character = kerning_style_part_one[kerning_loop]
                    kerning_width = layer_exists.width
                    kerning_y = int(layer_exists.height/2)
                    for kerning_character_loop in range(0,len(kerning_style_part_two[kerning_loop/3] ),2):
                        kerning_character_2 = kerning_style_part_two[kerning_loop/3][kerning_character_loop]
                        kerning_character_2_exists = pdb.gimp_image_get_layer_by_name(image, kerning_character_2 + ".png") ####
                        if kerning_character_2_exists: 
                            new_vector_name = kerning_character + kerning_character_2 + ".png" ####
						
                            if kern_style == 0:
                                #new_vector_name = kerning_character + kerning_character_2
                                kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair path
                                if kerning_vector>=0: 
                                
                                    NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(kerning_vector) # get vector information for kerning path width
                                    ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                                    kerning_target_width = pdb.gimp_vectors_stroke_get_length(kerning_vector,ActivePath_StrokeID,0.01)
                                else:
                                    kerning_target_width = 0
                            else:
                                kerning_target_width = float((kerning_width/100.00)) * float((kerning_style_part_two[kerning_loop/3][kerning_character_loop+1]))
                                kerning_target_width = int(kerning_target_width)
                            if kerning_target_width > 0:
                                kerning_target_width += fn_check_kern_adjustments(use_pixels, kerning_character_2, kerning_target_width, kerning_group_1, kerning_group_2, kern_g1_amount, kern_g2_amount, A_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern)
                                #new_vector_name = kerning_character + kerning_character_2
                                kerning_vector = pdb.gimp_image_get_vectors_by_name(image, new_vector_name) # check for character kerning pair paths
                                if kerning_vector>=0: # replace any existing accented character kerning path with a new version of the path
                                    pdb.gimp_image_remove_vectors(image, kerning_vector)
                                new_vector_id = pdb.gimp_vectors_new(image, new_vector_name)
                                pdb.gimp_image_insert_vectors(image, new_vector_id, None, -1)
                                points1 = [0,kerning_y] * 3
                                points2 = [kerning_target_width,kerning_y] * 3
                                points = points1 + points2
                                new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
						
    if single_letter == 26 and do_kern == False and do_kern_single == True:
        do_kern_single = False
        do_all_examples = True		
				
    if do_all_examples:
        plugin = False
        num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("Font Slab")
        if not "Font-Slab" in str(plugin_real_name):
            message = ("This option needs FontSlab version 0.12 (or greater) to be installed " + str(plugin_real_name) )
        else:
            version = str(plugin_real_name[len(plugin_real_name)-1])
            ptr = version.find("v")
            version_major = version[ptr + 1]
            version_minor = version[ptr + 3: len(version)]
            if int(version_major)>=0 and int(version_minor)>=12:
                #pdb.gimp_message("Font Slab version/s found: " + version + " : " + version_major + " : " + version_minor)
                plugin = True
            else:
                message = ("This option needs FontSlab version 0.12 (or greater) to be installed")

        if plugin == False:
            do_all_examples = False # Report no FontSlab
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message(message)	
            pdb.gimp_message_set_handler(old_handler)
			
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        if num_layers<5: do_all_examples = False
			
			
			
    if do_all_examples:
        # Creates a layer stack of all kerning pairs for each letter
        # set variables
        letters = ['Z','Y','X','W','V','U','T','S','R','Q','P','O','N','M','L','K','J','I','H','G','F','E','D','C','B','A']
        reversed_letters = list(letters)
        reversed_letters.reverse()
		
        file_name = pdb.gimp_image_get_filename(image) # get filename associated with this (custom font) image
				
        #rows_of_letters = 5
        #columns_of_letters = 6
        rows_of_letters = 6
        columns_of_letters = 5
		
		# check available fonts
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        our_font = font_list[0]
        if "Arial" in font_list: our_font = "Arial"
		
        path_x,path_x0,path_x1,path_len = 20,20,980,980-20 # path position values to align pairs
        
		# work out font size to use
        fontimg = pdb.gimp_xcf_load(-1,file_name,file_name)
		
        loop,fontimg_width = 0,0
        while loop<len(reversed_letters): # work out suitable font size to display kerning pairs
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,reversed_letters[loop]+".png")
            if char_layer.width>fontimg_width: fontimg_width,fontimg_height = char_layer.width,char_layer.height
            loop+=1
        fontimg_scale = float((float(path_len/columns_of_letters)/float(fontimg_width)))
        font_size = ((fontimg_scale * float(fontimg_height)/2)/1.164)
        font_size = (font_size / 100) * 90 # allow padding of 10% for biggest characters
        #pdb.gimp_message("font_size = " +  str(font_size) + " : Path to fit = " + str(path_len//columns_of_letters) + " : Max char width = " + str(fontimg_width ) + " : fontimg_scale = " + str(fontimg_scale) )
		
        # work out height for new image and create it
        #image_height = (columns_of_letters-1) * font_size + (columns_of_letters//2) * font_size
        image_height = 800
        new_image = pdb.gimp_image_new(1000,image_height,0)	
        status = pdb.gimp_image_undo_disable(new_image)	
		
        new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, "background layer", 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        new_display = pdb.gimp_display_new(new_image)
        pdb.gimp_item_set_visible(new_layer, False)
        bg = fn_get_bg_colour(bg_col)
        pdb.gimp_context_set_background(bg)
		
        for first_char_loop in range (len(reversed_letters)): #Create kerning pairs starting with Z combinations to end up with A combinations at top of layer stack
            current_char = letters[first_char_loop] # ie first letter in kerning pair
            new_layer_name = current_char + " kerning"
            new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, new_layer_name, 100, 0,) # background layer for current set of characters 
            pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
            pdb.gimp_drawable_fill(new_layer , 1)
		
            # get lists of vector ids in new image
            num_img_vectors, img_vector_ids = pdb.gimp_image_get_vectors(image)
            img_vector_list = image.vectors
            opacity = 50
 
            for loop in range(len(reversed_letters)): # create paths and add add kerning pairs using FontSlab plugin

                path_y = (font_size) + ((loop//columns_of_letters) * (font_size * 1.25))
                path_x0 = path_x + ((path_len//columns_of_letters)* (loop%columns_of_letters))
                new_vector_name = current_char + " path " + str((loop + 1))
                new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
                pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
                points1 = [path_x0,path_y] * 3
                points2 = [path_x1,path_y] * 3
                points = points1 + points2
                new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
			
                letter_group = current_char + reversed_letters[loop]
			
                slab_x = (path_len//rows_of_letters)*(loop//rows_of_letters)
                pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, font_size, False, False, "dummy", False, file_name, True, 0, letter_group, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
                active_layer = pdb.gimp_image_get_active_layer(new_image)
                kern_vector = pdb.gimp_image_get_vectors_by_name(image,letter_group + ".png")
                if kern_vector<0: pdb.gimp_layer_set_opacity(active_layer,opacity) # reduce opacity if there is no kerning path for the current characters
                new_layer = pdb.gimp_image_merge_down(new_image,active_layer,0)
					
            # get lists of vector ids in new image
            num_vectors, vector_ids = pdb.gimp_image_get_vectors(new_image)
            vector_list = new_image.vectors
            loop = 0
		
            while loop<num_vectors: # remove temp text paths
                vector_id = vector_list[loop]
                active_vector_name = pdb.gimp_item_get_name(vector_id)
                active_vector_name = active_vector_name.decode('utf-8')
                pdb.gimp_image_remove_vectors(new_image, vector_list[loop])
                loop += 1
		
        status = pdb.gimp_image_undo_enable(new_image)
				
				
				
    if do_user_kern:
        
        file_name = pdb.gimp_image_get_filename(image)
        if file_name>"":
		
            plugin = False
            num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("Font Slab")
            if not "Font-Slab" in str(plugin_real_name):
                message = ("This option needs FontSlab version 0.12 (or greater) to be installed " + str(plugin_real_name) )
            else:
                version = str(plugin_real_name[len(plugin_real_name)-1])
                ptr = version.find("v")
                version_major = version[ptr + 1]
                version_minor = version[ptr + 3: len(version)]
                if int(version_major)>=0 and int(version_minor)>=12:
                    #pdb.gimp_message("Font Slab version/s found: " + version + " : " + version_major + " : " + version_minor)
                    plugin = True
                else:
                    message = ("This option needs FontSlab version 0.12 (or greater) to be installed")

            if plugin == False:
                do_user_kern = False # Report no FontSlab
                old_handler = pdb.gimp_message_get_handler()		
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message(message)	
                pdb.gimp_message_set_handler(old_handler)
				
        else:
            do_user_kern = False # Report no File
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message("Cannot find a custom font associated with this image.")
            pdb.gimp_message_set_handler(old_handler)
            
            
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        if num_layers<5: do_user_kern = False
			
    if do_user_kern: # displays user string using current kerning
        kern_string.lstrip()
        if kern_string == "" or kern_string.isspace(): kern_string = "example text"
        kern_string = kern_string.upper()
		
			
        file_name = pdb.gimp_image_get_filename(image)
        new_image = pdb.gimp_image_new(1000,700,0)
        status = pdb.gimp_image_undo_disable(new_image)
        new_layer = pdb.gimp_layer_new(new_image, 1000, 700, 1, "background layer", 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        new_display = pdb.gimp_display_new(new_image)
        pdb.gimp_item_set_visible(new_layer, False)
		
        path_x = 980
		
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        our_font = font_list[0]
        if "Arial" in font_list: our_font = "Arial"
		
        if len(kern_string)>12:
            no_of_spaces = kern_string.count(" ")
            #pdb.gimp_message("no_of_spaces = " + str(no_of_spaces) )
            if no_of_spaces>0:
                cut_off = 1
                if no_of_spaces>1:
                    cut_off = no_of_spaces//2
                    if no_of_spaces%2>0: cut_off += 1
                    loop,ctr = 0,0
                    while loop<len(kern_string):
                        char = kern_string[loop]
                        if char == " ": ctr += 1
                        if ctr == cut_off:
                            kern1 = kern_string[0:loop]
                            kern2 = kern_string[loop + 1:len(kern_string)]
                            loop = len(kern_string)
                        loop+=1
                else:
                    loop = kern_string.find(" ")
                    kern1 = kern_string[0:loop]
                    kern2 = kern_string[loop + 1:len(kern_string)]
            else:
                loop = len(kern_string)//2
                if len(kern_string)%2>0:loop+=1
                kern1 = kern_string[0:loop]
                kern2 = kern_string[loop:len(kern_string)]
				
            path_y = 150
			
            new_vector_name = "example path 1"
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [20,path_y] * 3
            points2 = [path_x,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		
            new_layer_name = "example layer 1"
            new_layer = pdb.gimp_layer_new(new_image, 1000, 700, 1, new_layer_name, 100, 0,)
            pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
            bg = fn_get_bg_colour(bg_col)
            pdb.gimp_context_set_background(bg)
            pdb.gimp_drawable_fill(new_layer , 1)
		
		
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, 100, True, False, "dummy", False, file_name, True, 0, kern1, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
		
            #new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
				
				
            path_y = 450
			
            new_vector_name = "example path 2"
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [20,path_y] * 3
            points2 = [path_x,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		
            new_layer_name = "example layer 2"		
		
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, 100, True, False, "dummy", False, file_name, True, 0, kern2, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
		
            new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
				
        else:           
		
            path_y = 300
            new_vector_name = "example path "
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [20,path_y] * 3
            points2 = [path_x,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
		
            new_layer_name = "example layer"
            new_layer = pdb.gimp_layer_new(new_image, 1000, 700, 1, new_layer_name, 100, 0,)
            pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
            bg = fn_get_bg_colour(bg_col)
            pdb.gimp_context_set_background(bg)
            pdb.gimp_drawable_fill(new_layer , 1)
		
		
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, 100, True, False, "dummy", False, file_name, True, 0, kern_string, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
		
            new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
					
        # get lists of vector ids in new image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(new_image)
        vector_list = new_image.vectors
        loop = 0
        while loop<num_vectors: # remove paths from the newly created image
            vector_id = vector_list[loop]
            active_vector_name = pdb.gimp_item_get_name(vector_id)
            active_vector_name = active_vector_name.decode('utf-8')
            pdb.gimp_image_remove_vectors(new_image, vector_list[loop])
            loop += 1
		
        status = pdb.gimp_image_undo_enable(new_image)
		
					
    if do_kern_single:
        # displays a single page of kerning pairs for the selected letter
        plugin = False
        num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("Font Slab")
        if not "Font-Slab" in str(plugin_real_name):
            message = ("This option needs FontSlab version 0.12 (or greater) to be installed " + str(plugin_real_name) )
        else:
            version = str(plugin_real_name[len(plugin_real_name)-1])
            ptr = version.find("v")
            version_major = version[ptr + 1]
            version_minor = version[ptr + 3: len(version)]
            if int(version_major)>=0 and int(version_minor)>=12:
                #pdb.gimp_message("Font Slab version/s found: " + version + " : " + version_major + " : " + version_minor)
                plugin = True
            else:
                message = ("This option needs FontSlab version 0.12 (or greater) to be installed")
				
        file_name = pdb.gimp_image_get_filename(image)
        #pdb.gimp_message("file_name = " + str(file_name))
        if file_name == "": plugin = False

        if plugin == False:
            do_kern_single = False # Report no FontSlab
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message(message)	
            pdb.gimp_message_set_handler(old_handler)
			
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        if num_layers<5: do_kern_single = False
			
    if do_kern_single:
		
        letters = ['Z','Y','X','W','V','U','T','S','R','Q','P','O','N','M','L','K','J','I','H','G','F','E','D','C','B','A']
        reversed_letters = list(letters)
        reversed_letters.reverse()
		
        file_name = pdb.gimp_image_get_filename(image)
        
		
        #rows_of_letters = 5
        #columns_of_letters = 6
        rows_of_letters = 6
        columns_of_letters = 5
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        our_font = font_list[0]
        if "Arial" in font_list: our_font = "Arial"
		
        path_x,path_x0,path_x1,path_len = 20,20,980,980-20
        
        fontimg = pdb.gimp_xcf_load(-1,file_name,file_name)
        loop,fontimg_width = 0,0
        while loop<len(reversed_letters):
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,reversed_letters[loop]+".png")
		
        
            if char_layer.width>fontimg_width: fontimg_width,fontimg_height = char_layer.width,char_layer.height
            loop+=1
        fontimg_scale = float((float(path_len/columns_of_letters)/float(fontimg_width)))
        font_size = ((fontimg_scale * float(fontimg_height)/2)/1.164)
        font_size = (font_size / 100) * 90 # allow padding of 10% for biggest characters
        #pdb.gimp_message("font_size = " +  str(font_size) + " : Path to fit = " + str(path_len//columns_of_letters) + " : Max char width = " + str(fontimg_width ) + " : fontimg_scale = " + str(fontimg_scale) )
		
		
        #image_height = (columns_of_letters-1) * font_size + (columns_of_letters//2) * font_size
        image_height = 800
        new_image = pdb.gimp_image_new(1000,image_height,0)
        status = pdb.gimp_image_undo_disable(new_image)
        new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, "background layer", 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        new_display = pdb.gimp_display_new(new_image)
        pdb.gimp_item_set_visible(new_layer, False)
        bg = fn_get_bg_colour(bg_col)
        pdb.gimp_context_set_background(bg)
		
        current_char = reversed_letters[single_letter]
        new_layer_name = current_char + " kerning"
        new_layer = pdb.gimp_layer_new(new_image, 1000, image_height, 1, new_layer_name, 100, 0,)
        pdb.gimp_image_insert_layer(new_image, new_layer, None, -1)
        pdb.gimp_drawable_fill(new_layer , 1)
		
		
		
        # get lists of vector ids in new image
        num_img_vectors, img_vector_ids = pdb.gimp_image_get_vectors(image)
        img_vector_list = image.vectors
        opacity = 50
 
        for loop in range(len(reversed_letters)):

            #path_y = (font_size) + ((loop//columns_of_letters) * (font_size * 1.5))
            path_y = (font_size) + ((loop//columns_of_letters) * (font_size * 1.25))
            path_x0 = path_x + ((path_len//columns_of_letters)* (loop%columns_of_letters))
            new_vector_name = current_char + " path " + str((loop + 1))
            new_vector_id = pdb.gimp_vectors_new(new_image, new_vector_name)
            pdb.gimp_image_insert_vectors(new_image, new_vector_id, None, -1)
            points1 = [path_x0,path_y] * 3
            points2 = [path_x1,path_y] * 3
            points = points1 + points2
            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
			
            letter_group = current_char + reversed_letters[loop]
			
            slab_x = (path_len//rows_of_letters)*(loop//rows_of_letters)
            #font_size = 94 - 10
            pdb.python_fu_Font_Slab_v0_12(new_image, new_layer, our_font, font_size, False, False, "dummy", False, file_name, True, 0, letter_group, False, False, 0, False, 100, 200, 100, 50, False, False, False, 0, 0, False)
            active_layer = pdb.gimp_image_get_active_layer(new_image)
            kern_vector = pdb.gimp_image_get_vectors_by_name(image,letter_group + ".png")
            if kern_vector<0: pdb.gimp_layer_set_opacity(active_layer,opacity) # reduce opacity if there is no kerning path for the current characters
					
        # get lists of vector ids in new image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(new_image)
        vector_list = new_image.vectors
        loop = 0
        while loop<num_vectors: # remove paths from the newly created image
            vector_id = vector_list[loop]
            active_vector_name = pdb.gimp_item_get_name(vector_id)
            active_vector_name = active_vector_name.decode('utf-8')
            pdb.gimp_image_remove_vectors(new_image, vector_list[loop])
            loop += 1
		
        new_layer = pdb.gimp_image_merge_visible_layers(new_image,0)
        status = pdb.gimp_image_undo_enable(new_image)

		
    if do_finalize:					
        # get lists of vector ids in new image
        num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
        vector_list = image.vectors
        loop = 0
        while loop<num_vectors:
            vector_id = vector_list[loop]
            active_vector_name = pdb.gimp_item_get_name(vector_id) #name of the current vector
            active_vector_name = active_vector_name.decode('utf-8')
            if len(active_vector_name)==6 and ".png" in active_vector_name: # only work with kerning pairs
                test_layer = active_vector_name[0] # get first character of kerning pair name
                test_layer+=".png" # add custom font extension to get layer name 
                test_layer_id=pdb.gimp_image_get_layer_by_name(image,test_layer) # get layer id
                NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(vector_id) # get vector information for kerning path width
                ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                kern_width = pdb.gimp_vectors_stroke_get_length(vector_id,ActivePath_StrokeID,0.01)
                if test_layer_id.width==kern_width: # only remove vectors if they are the same width as the actual layer width
                    pdb.gimp_image_remove_vectors(image, vector_list[loop])
            loop += 1

    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

def fn_get_bg_colour(bg_col):
    if bg_col == 0: colour = black
    elif bg_col == 1: colour = white
    elif bg_col == 2: colour = pdb.gimp_context_get_foreground()
    elif bg_col == 3: colour = pdb.gimp_context_get_background()
    else:
        colour = black
    return colour
	
######################################################################################################################################

def fn_check_kern_adjustments(use_pixels, kerning_character_2, kerning_character_width, kerning_group_1, kerning_group_2, kern_g1_amount, kern_g2_amount, A_kern, J_kern, S_kern, T_kern, U_kern, V_kern, W_kern, X_kern, Y_kern, Z_kern):
    kern_adjustment = 0
    if kerning_character_2 in kerning_group_1:
        if use_pixels:
            kern_adjustment = abs(kern_g1_amount)
        else:
            if kern_g1_amount< -95: kern_g1_amount = -95
            kern_adjustment = (float(kerning_character_width/100) * abs(kern_g1_amount))
        if kern_g1_amount<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 in kerning_group_2:
        if use_pixels:
            kern_adjustment = abs(kern_g2_amount)
        else:
            if kern_g2_amount< -95: kern_g2_amount = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(kern_g2_amount))
        if kern_g2_amount<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "A":
        if use_pixels:
            kern_adjustment = abs(A_kern)
        else:
            if A_kern< -95: A_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(A_kern))
        if A_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "J":
        if use_pixels:
            kern_adjustment = abs(J_kern)
        else:
            if J_kern< -95: J_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(J_kern))
        if J_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "S":  
        if use_pixels:
            kern_adjustment = abs(S_kern)
        else:
            if S_kern< -95: S_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(S_kern))
        if S_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "T":
        if use_pixels:
            kern_adjustment = abs(T_kern)
        else:
            if T_kern< -95: T_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(T_kern))
        if T_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "U":
        if use_pixels:
            kern_adjustment = abs(U_kern)
        else:
            if U_kern< -95: U_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(U_kern))
        if U_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "V":
        if use_pixels:
            kern_adjustment = abs(V_kern)
        else:
            if V_kern< -95: V_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(V_kern))
        if V_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "W":
        if use_pixels:
            kern_adjustment = abs(W_kern)
        else:
            if W_kern< -95: W_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(W_kern))
        if W_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "X":
        if use_pixels:
            kern_adjustment = abs(X_kern)
        else:
            if X_kern< -95: X_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(X_kern))
        if X_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "Y":
        if use_pixels:
            kern_adjustment = abs(Y_kern)
        else:
            if Y_kern< -95: Y_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(Y_kern))
        if Y_kern<0: kern_adjustment = 0 - kern_adjustment
    if kerning_character_2 == "Z":
        if use_pixels:
            kern_adjustment = abs(Z_kern)
        else:
            if Z_kern< -95: Z_kern = -95
            kern_adjustment = (float(kerning_character_width/100) *  abs(Z_kern))
        if Z_kern<0: kern_adjustment = 0 - kern_adjustment
    return kern_adjustment
    

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Kerning_UpperCase_v003b",
    "Provides options for creating kerning paths for upper-case letters in custom fonts ",
    "Provides options for creating kerning paths for upper-case letters in custom fonts ",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2019",
    "<Image>/Filters/Custom Font Tools/Kerning Tools/Custom Font Kerning for Uppercase v003b...",
    "RGBA",
    [
    (PF_TOGGLE, "kern_sans", "Create upper case kerning paths for custom font: ", 0),
    (PF_OPTION, "kern_style", "Kerning style used: ", 1, ("Use kerning paths if available","Common pairs","Sans Serif","Serif","All pairs")),
    (PF_OPTION, "kern_current", "Apply kerning to: ", 0, ("All letters","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z")),
    (PF_TOGGLE, "use_pix", "Make adjustments in pixels: ", 0),
    (PF_SPINNER, "spin_adjust_g1_caps", "Group 1 upper case adjustment (%): \n ie B D E F H I K L M N P R", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_g2_caps", "Group 2 upper case adjustment (%): \n ie C G O Q", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_A_cap", "Upper case A adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_J_cap", "Upper case J adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_S_cap", "Upper case S adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_T_cap", "Upper case T adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_U_cap", "Upper case U adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_V_cap", "Upper case V adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_W_cap", "Upper case W adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_X_cap", "Upper case X adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_Y_cap", "Upper case Y adjustment (%): ", 0,(-500,500,1)),
    (PF_SPINNER, "spin_adjust_Z_cap", "Upper case Z adjustment (%): ", 0,(-500,500,1)),
    (PF_TOGGLE, "kern_user", "Show currently saved kerning in\nthe text below (new image): ", 0),
    (PF_STRING, "kern_string", "Kerning Text:", "The very quick brown fox jumps over the lazy dog."),
    (PF_TOGGLE, "kern_example", "Select one current kerning example \nwith the currently saved kerning (new image): ", 0),
    (PF_OPTION, "kern_one", "Show kerning for: ", 0, ("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","All letters")),
    (PF_OPTION, "kern_bg", "Background colour for kerning examples: ", 0, ("Black","White","Current foreground colour","Current background colour")),
    (PF_TOGGLE, "finalize", "Finalize kerning paths: ", 0),
    ],
    [],
    do_stuff)

main()
